//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  SocketCAN header translation by Zeljko Avramovic                                    //
//  More info can be found at https://www.kernel.org/doc/html/v5.10/networking/can.html //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

unit can.bcm;

{$packrecords c}

interface

uses
  ctypes, can;


(* SPDX-License-Identifier: ((GPL-2.0-only WITH Linux-syscall-note) OR BSD-3-Clause) *)
(*
 * linux/can/bcm.h
 *
 * Definitions for CAN Broadcast Manager (BCM)
 *
 * Author: Oliver Hartkopp <oliver.hartkopp@volkswagen.de>
 * Copyright (c) 2002-2007 Volkswagen Group Electronic Research
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Volkswagen nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * Alternatively, provided that this notice is retained in full, this
 * software may be distributed under the terms of the GNU General
 * Public License ('GPL') version 2, in which case the provisions of the
 * GPL apply INSTEAD OF those given above.
 *
 * The provided data structures and external interfaces from this code
 * are not restricted to be used by modules with a GPL compatible license.
 *
 *  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING  ANY WAY  OF THE USE
 * OF  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *)

type
  bcm_timeval = record
    tv_sec:  clong;
    tv_usec: clong;
  end;

  (*
   *  bcm_msg_head - head of messages to/from the broadcast manager
   * @opcode:    opcode, see enum below.
   * @flags:     special flags; see below.
   * @count:     number of frames to send before changing interval.
   * @ival1:     interval for the first @count frames.
   * @ival2:     interval for the following frames.
   * @can_id:    CAN ID of frames to be sent or received.
   * @nframes:   number of frames appended to the message head.
   * @frames:    aArray of CAN frames.
   *)

  // bcm_msg_head for CAN frames
  bcm_msg_head_can_frame = record
    opcode:  cuint32;
    flags:   cuint32;
    count:   cuint32;
    ival1:   bcm_timeval;
    ival2:   bcm_timeval;
    can_id:  canid_t;
    nframes: cuint32;
    frames:  array[0..0] of can_frame;
  end;

  bcm_msg_head = bcm_msg_head_can_frame;  // C examples from the net mostly use this name

  bcm_msg_head_without_can_frame = record // work around added by Avra
    opcode:  cuint32;
    flags:   cuint32;
    count:   cuint32;
    ival1:   bcm_timeval;
    ival2:   bcm_timeval;
    can_id:  canid_t;
    nframes: cuint32;
    // frames:  array[0..0] of can_frame; // frames[] removed because for BCM cyclic demo when more then 1 frame is needed we already
  end;                                    // had frames[0] in original bcm_msg_head therefore we had to populate 2 frame structures

  // bcm_msg_head for CAN FD frames
  bcm_msg_head_canfd_frame = record       // work around added by Avra
    opcode:  cuint32;
    flags:   cuint32;
    count:   cuint32;
    ival1:   bcm_timeval;
    ival2:   bcm_timeval;
    can_id:  canid_t;
    nframes: cuint32;
    frames:  array[0..0] of canfd_frame;
  end;

  bcm_msg_head_without_canfd_frame = record // work around added by Avra
    opcode:  cuint32;
    flags:   cuint32;
    count:   cuint32;
    ival1:   bcm_timeval;
    ival2:   bcm_timeval;
    can_id:  canid_t;
    nframes: cuint32;
    // frames:  array[0..0] of can_fdframe; // frames[] removed because for BCM cyclic demo when more then 1 frame is needed we already
  end;                                      // had frames[0] in original bcm_msg_head therefore we had to populate 2 frame structures

const
  TX_SETUP   =  1;  (* create (cyclic) transmission task                 *)
  TX_DELETE  =  2;  (* remove (cyclic) transmission task                 *)
  TX_READ    =  3;  (* read properties of (cyclic) transmission task     *)
  TX_SEND    =  4;  (* send one CAN frame                                *)
  RX_SETUP   =  5;  (* create RX content filter subscription             *)
  RX_DELETE  =  6;  (* remove RX content filter subscription             *)
  RX_READ    =  7;  (* read properties of RX content filter subscription *)
  TX_STATUS	 =  8;  (* reply to TX_READ request                          *)
  TX_EXPIRED =  9;  (* notification on performed transmissions (count=0) *)
  RX_STATUS  = 10;  (* reply to RX_READ request                          *)
  RX_TIMEOUT = 11;  (* cyclic message is absent                          *)
  RX_CHANGED = 12;  (* updated CAN frame (detected content change)       *)

  SETTIMER           = $0001;
  STARTTIMER         = $0002;
  TX_COUNTEVT        = $0004;
  TX_ANNOUNCE        = $0008;
  TX_CP_CAN_ID       = $0010;
  RX_FILTER_ID       = $0020;
  RX_CHECK_DLC       = $0040;
  RX_NO_AUTOTIMER    = $0080;
  RX_ANNOUNCE_RESUME = $0100;
  TX_RESET_MULTI_IDX = $0200;
  RX_RTR_FRAME       = $0400;
  CAN_FD_FRAME       = $0800;

implementation

end.

