//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  SocketCAN header translation by Zeljko Avramovic                                    //
//  More info can be found at https://www.kernel.org/doc/html/v5.10/networking/can.html //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

unit can.netlink;

{$mode objfpc}
{$packrecords c}

interface

uses
  ctypes;


(* SPDX-License-Identifier: GPL-2.0-only WITH Linux-syscall-note *)
(*
 * linux/can/netlink.h
 *
 * Definitions for the CAN netlink interface
 *
 * Copyright (c) 2009 Wolfgang Grandegger <wg@grandegger.com>
 *
 * This aProgram is free software; you can redistribute it and/or modify
 * it under the terms of the version 2 of the GNU General Public License
 * as published by the Free Software Foundation
 *
 * This aProgram is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *)

type

  // CAN bit-timing parameters
  //
  // For further information, please read chapter '8 BIT TIMING
  // REQUIREMENTS' of the 'Bosch CAN Specification version 2.0'
  // at http://www.semiconductors.bosch.de/pdf/can2spec.pdf.
  can_bittiming = record
    bitrate:      cuint32;  // Bit-rate in bits/second
    sample_point: cuint32;  // Sample point in one-tenth of a percent
    tq:           cuint32;  // Time quanta (TQ) in nanoseconds
    prop_seg:     cuint32;  // Propagation segment in TQs
    phase_seg1:   cuint32;  // Phase buffer segment 1 in TQs
    phase_seg2:   cuint32;  // Phase buffer segment 2 in TQs
    sjw:          cuint32;  // Synchronisation jump width in TQs
    brp:          cuint32;  // Bit-rate prescaler
  end;

  // CAN hardware-dependent bit-timing constant
  //
  // Used for calculating and checking bit-timing parameters
  can_bittiming_const = record
    name:      array[0..15] of char;  // Name of the CAN controller hardware
    tseg1_min: cuint32;               // Time segment 1 = prop_seg + phase_seg1
    tseg1_max: cuint32;
    tseg2_min: cuint32;               // Time segment 2 = phase_seg2
    tseg2_max: cuint32;
    sjw_max:   cuint32;               // Synchronisation jump width
    brp_min:   cuint32;               // Bit-rate prescaler
    brp_max:   cuint32;
    brp_inc:   cuint32;
  end;

  // CAN clock parameters
  can_clock = record
    freq: cuint32;  // CAN system clock frequency in Hz
  end;

const
  // CAN operational and error states
  CAN_STATE_ERROR_ACTIVE  = 0;  // RX/TX error count < 96
  CAN_STATE_ERROR_WARNING = 1;  // RX/TX error count < 128
  CAN_STATE_ERROR_PASSIVE = 2;  // RX/TX error count < 256
  CAN_STATE_BUS_OFF       = 3;  // RX/TX error count >= 256
  CAN_STATE_STOPPED       = 4;  // Device is stopped
  CAN_STATE_SLEEPING      = 5;  // Device is sleeping
  CAN_STATE_MAX           = 6;

type
	can_state = CAN_STATE_ERROR_ACTIVE..CAN_STATE_MAX;

  // CAN bus error counters
  can_berr_counter = record
    txerr: cuint16;
    rxerr: cuint16;
  end;

  // CAN controller mode
	can_ctrlmode = record
    mask: cuint32;
    flags: cuint32;
  end;

const
  CAN_CTRLMODE_LOOPBACK       = $01;  // Loopback mode
  CAN_CTRLMODE_LISTENONLY     = $02;  // Listen-only mode
  CAN_CTRLMODE_3_SAMPLES      = $04;  // Triple sampling mode
  CAN_CTRLMODE_ONE_SHOT       = $08;  // One-Shot mode
  CAN_CTRLMODE_BERR_REPORTING = $10;  // Bus-error reporting
  CAN_CTRLMODE_FD             = $20;  // CAN FD mode
  CAN_CTRLMODE_PRESUME_ACK    = $40;  // Ignore missing CAN ACKs
  CAN_CTRLMODE_FD_NON_ISO     = $80;  // CAN FD in non-ISO mode

type
  // CAN device statistics
  can_device_stats = record
    bus_error:        cuint32;  // Bus errors
    error_warning:    cuint32;  // Changes to error warning state
    error_passive:    cuint32;  // Changes to error passive state
    bus_off:          cuint32;  // Changes to bus off state
    arbitration_lost: cuint32;  // Arbitration lost errors
    restarts:         cuint32;  // CAN controller re-starts
  end;

const
  // CAN netlink interface
  IFLA_CAN_UNSPEC               = 0;
  IFLA_CAN_BITTIMING            = 1;
  IFLA_CAN_BITTIMING_CONST      = 2;
  IFLA_CAN_CLOCK                = 3;
  IFLA_CAN_STATE                = 4;
  IFLA_CAN_CTRLMODE             = 5;
  IFLA_CAN_RESTART_MS           = 6;
  IFLA_CAN_RESTART              = 7;
  IFLA_CAN_BERR_COUNTER         = 8;
  IFLA_CAN_DATA_BITTIMING       = 9;
  IFLA_CAN_DATA_BITTIMING_CONST	= 10;
  IFLA_CAN_TERMINATION          = 11;
  IFLA_CAN_TERMINATION_CONST    = 12;
  IFLA_CAN_BITRATE_CONST        = 13;
  IFLA_CAN_DATA_BITRATE_CONST   = 14;
  IFLA_CAN_BITRATE_MAX          = 15;
  __IFLA_CAN_MAX                = 16;

  IFLA_CAN_MAX = __IFLA_CAN_MAX - 1;

  // u16 termination range: 1..65535 Ohms
  CAN_TERMINATION_DISABLED = 0;

implementation

end.

