//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN BCM cyclic demo created by Zeljko Avramovic             //
//  FPC modified LGPL licence: https://wiki.freepascal.org/FPC_modified_LGPL    //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program cancyclic;

{$mode delphi} {$H+}

uses
  classes, sysutils, sockets, baseunix, crt, can, can.bcm;

type
  bcm_msg = record
    head: bcm_msg_head_without_can_frame; // we shall use more then 1 frame so bcm_msg_head_can_frame is not enough
    frame: array[0..2] of can_frame;      // and we add here space for our 3 frames
  end;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  s, i: cint;
  addr: sockaddr_can;
  ifr:  ifreq;
  msg:  bcm_msg;
begin
  try
    WriteLn('CAN Sockets BCM Cyclic Demo');
    s := fpsocket(PF_CAN, SOCK_DGRAM, CAN_BCM);
    if s < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    strcopy(ifr.ifr_name, 'vcan0'); // ifr.ifr_name := 'vcan0' + #0;
    if fpioctl(s, SIOCGIFINDEX, @ifr) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    memset(@addr, 0, sizeof(addr));
    addr.can_family  := AF_CAN;
    addr.can_ifindex := ifr.ifr_ifindex;

    if fpconnect(s, @addr, sizeof(addr)) < 0 then
    begin
      perror('Connect');
      Exit;
    end;

    msg.head.opcode        := TX_SETUP;
    msg.head.can_id        := $567; // if TX_CP_CAN_ID flag is set then all messages shall have this CAN ID
    msg.head.flags         := SETTIMER OR STARTTIMER OR TX_ANNOUNCE {OR TX_CP_CAN_ID} {OR TX_RESET_MULTI_IDX};
    msg.head.nframes       := 3;
    msg.head.count         := 0; // single interval (use only ival2)
    msg.head.ival1.tv_sec  := 0;
    msg.head.ival1.tv_usec := 0;
    msg.head.ival2.tv_sec  := 0;
    msg.head.ival2.tv_usec := 250000; // 250 ms interval between CAN messages
    msg.frame[0].can_id    := $550; // not used if TX_CP_CAN_ID falg is set
    msg.frame[0].can_dlc   := 2;
    msg.frame[0].data      := [$01,$02,$03,$04,$05,$06,$07,$08];
    msg.frame[1].can_id    := $551; // not used if TX_CP_CAN_ID falg is set
    msg.frame[1].can_dlc   := 4;
    msg.frame[1].data      := [$11,$12,$13,$14,$15,$16,$17,$18];
    msg.frame[2].can_id    := $552; // not used if TX_CP_CAN_ID falg is set
    msg.frame[2].can_dlc   := 8;
    msg.frame[2].data      := [$21,$22,$23,$24,$25,$26,$27,$28];

    if fpwrite(s, msg, sizeof(msg)) <> sizeof(msg) then
    begin
      perror('Write');
      Exit;
    end;

    repeat
      WriteLn('Press any key to stop...');
      msg.frame[2].DataByte[0] := msg.frame[2].DataByte[0] + 1;
      for i := 0 to msg.frame[2].can_dlc - 1 do
        Write(Format('%02.2X ', [msg.frame[2].data[i]]));
      WriteLn('');
      WriteLn('');
      fpwrite(s, msg, sizeof(msg)); // comment out this line if you do not want to publish changed frame[2]
      Delay(2000); // loop every 2 seconds
    until KeyPressed;

    if fpclose(s) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

